/*******************************************************************************
 * Copyright (c) 2020 THALES GLOBAL SERVICES.
 * 
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *    Thales - initial API and implementation
 *******************************************************************************/
package org.polarsys.capella.transition.system2subsystem.rules.information.communication;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.polarsys.capella.core.data.cs.BlockArchitecture;
import org.polarsys.capella.core.data.cs.CsPackage;
import org.polarsys.capella.core.data.information.communication.CommunicationLink;
import org.polarsys.capella.core.data.information.communication.CommunicationPackage;
import org.polarsys.capella.core.model.helpers.BlockArchitectureExt;
import org.polarsys.capella.core.transition.common.constants.ITransitionConstants;
import org.polarsys.capella.core.transition.common.handlers.attachment.AttachmentHelper;
import org.polarsys.capella.core.transition.common.handlers.contextscope.ContextScopeHandlerHelper;
import org.polarsys.capella.core.transition.common.handlers.transformation.TransformationHandlerHelper;
import org.polarsys.capella.core.transition.system.rules.AbstractCapellaElementRule;
import org.polarsys.kitalpha.transposer.rules.handler.rules.api.IContext;
import org.polarsys.kitalpha.transposer.rules.handler.rules.api.IPremise;

public class CommunicationLinkRule extends AbstractCapellaElementRule {

  public CommunicationLinkRule() {
    registerUpdatedAttribute(CommunicationPackage.Literals.COMMUNICATION_LINK__KIND);
    registerUpdatedAttribute(CommunicationPackage.Literals.COMMUNICATION_LINK__PROTOCOL);
    registerUpdatedReference(CommunicationPackage.Literals.COMMUNICATION_LINK__EXCHANGE_ITEM);
  }

  @Override
  protected EClass getSourceType() {
    return CommunicationPackage.Literals.COMMUNICATION_LINK;
  }

  @Override
  public IStatus transformRequired(EObject source, IContext context) {
    IStatus result = super.transformRequired(source, context);

    if (result.isOK()) {
      CommunicationLink element = (CommunicationLink) source;
      EObject sourceElement = element.getExchangeItem();
      result = TransformationHandlerHelper.getInstance(context).checkTransformRequired(element, context, sourceElement);
    }
    return result;
  }

  @Override
  protected EObject getSourceContainer(EObject element, EObject result, IContext context) {
    return getSource(element, context);
  }

  protected EObject getSource(EObject source_p, IContext context_p) {
    CommunicationLink element = (CommunicationLink) source_p;
    return element.eContainer();
  }
  
  @Override
  protected EObject getDefaultContainer(EObject element, EObject result, IContext context) {
    EObject root = TransformationHandlerHelper.getInstance(context).getLevelElement(element, context);
    BlockArchitecture target =
        (BlockArchitecture) TransformationHandlerHelper.getInstance(context).getBestTracedElement(root, context, CsPackage.Literals.BLOCK_ARCHITECTURE,
            element, result);
    return BlockArchitectureExt.getOrCreateSystem(target);
  }

  @Override
  protected void retrieveGoDeep(EObject source, List<EObject> result, IContext context) {
    super.retrieveGoDeep(source, result, context);
    CommunicationLink link = (CommunicationLink) source;

    if (ContextScopeHandlerHelper.getInstance(context).contains(ITransitionConstants.SOURCE_SCOPE, source, context)) {
      result.add(link.getExchangeItem());
    }
  }

  @Override
  protected void premicesRelated(EObject element, ArrayList<IPremise> needed) {
    needed.addAll(createDefaultPrecedencePremices(element, CommunicationPackage.Literals.COMMUNICATION_LINK__EXCHANGE_ITEM));
  }

  @Override
  protected void attachRelated(EObject element, EObject result, IContext context) {
    AttachmentHelper.getInstance(context).attachTracedElements(element, result, CommunicationPackage.Literals.COMMUNICATION_LINK__EXCHANGE_ITEM,
        context);
  }
}

